﻿/**
 * Unityのエディタ機能でのペイントのテスト.
 * ※ DrawAreaTest.csはEditorフォルダに配置する.
 */

using UnityEngine;
using System.Collections;
using UnityEditor;

public class DrawAreaTest : EditorWindow {
	private Texture2D m_preview = null;				// プレビュー領域.
	private Color m_DrawColor   = Color.black;		// 描画色.
	private Vector2 m_startPos  = Vector2.zero;		// マウスドラッグ開始位置.
	private Vector2 m_endPos    = Vector2.zero;		// マウスドラッグ終了位置.
	private DrawTexture2D m_draw = null;			// 描画クラス.
	private float m_penSize = 1.0f;					// ペンサイズ.
	private Rect m_DrawArea = new Rect(8, 60, 256, 256);	// 描画領域.
	
	[MenuItem ("Window/DrawAreaTest")]
    static void Init() {
		DrawAreaTest window = (DrawAreaTest)EditorWindow.GetWindow(typeof(DrawAreaTest));
		window.Initialize();
		window.Show();
	}
	
	void Initialize() {
		m_preview = new Texture2D((int)m_DrawArea.width, (int)m_DrawArea.height, TextureFormat.RGBA32, false);
		m_draw = new DrawTexture2D();
		m_draw.Begin(m_preview);
		m_draw.Clear(Color.white);
		m_draw.End();
	}
	
	/**
	 * GUI部品の配置.
	 */
	void OnGUI() {
		wantsMouseMove = true;   // マウス情報を取得.

		EditorGUILayout.BeginVertical("box");
		EditorGUILayout.BeginHorizontal();
		if (GUILayout.Button("Clear")) {
			DrawClear();
		}
		EditorGUILayout.Space();
		m_DrawColor = EditorGUILayout.ColorField("Draw Color:", m_DrawColor);
		EditorGUILayout.EndHorizontal();
		m_penSize = EditorGUILayout.Slider("PenSize:", m_penSize, 1.0f, 5.0f);
		EditorGUILayout.Space();
		EditorGUILayout.EndVertical();
		
		if (m_preview != null) {
			float x = m_DrawArea.x;
			float y = m_DrawArea.y;
			float wid = m_preview.width;
			float hei = m_preview.height;
			EditorGUI.DrawPreviewTexture(new Rect(x, y, wid, hei), m_preview);
		}
		
		DrawMouseEvent();
	}
	
	/**
	 * 描画領域をクリアする.
	 */
	void DrawClear() {
		if (m_preview == null) return;
		m_draw.Begin(m_preview);
		m_draw.Clear(Color.white);
		m_draw.End();
	}
	
	void DrawMouseEvent() {
		Vector2 mousePos = Event.current.mousePosition;
		bool redrawF = false;
		
		// マウスダウン.
		if (Event.current.type == EventType.MouseDown) {
			m_startPos = m_endPos = mousePos;
		}
		
		// マウスドラッグ中.
		if (Event.current.type == EventType.MouseDrag && m_startPos != Vector2.zero) {
			m_endPos = mousePos;
			
			int hei = (int)m_preview.height;
			int x1 = (int)(m_startPos.x - m_DrawArea.x);
			int y1 = hei - (int)(m_startPos.y - m_DrawArea.y);
			int x2 = (int)(m_endPos.x   - m_DrawArea.x);
			int y2 = hei - (int)(m_endPos.y   - m_DrawArea.y);
			
			// 線をTexture2Dに描画する.
			m_draw.Begin(m_preview);
			m_draw.DrawLine(x1, y1, x2, y2, m_DrawColor, m_penSize);
			m_draw.End();
			
			m_startPos = m_endPos;
			redrawF = true;
		}
		
		// マウスアップ.
		if (Event.current.type == EventType.MouseUp) {
			m_startPos = m_endPos = Vector2.zero;
		}
		
		// 再描画を促す.
		if (Event.current.type == EventType.MouseMove || redrawF) {
			Repaint();
		}
	}
}
